"use strict";
require('reflect-metadata');
class Injector {
    constructor() {
        this.temporaryValue = Symbol.for('recursive-injection');
        this.cache = new Map();
    }
    resolveInjectable(ctorFn) {
        if (!ctorFn) {
            console.error('Attempted to inject a falsey type.');
            return;
        }
        if (this.cache.get(ctorFn) === this.temporaryValue) {
            console.error(`Recursive injection of type ${ctorFn.name || ctorFn}`);
            return;
        }
        if (this.cache.has(ctorFn)) {
            return this.cache.get(ctorFn);
        }
        this.cache.set(ctorFn, this.temporaryValue);
        let types = Reflect.getOwnMetadata('design:paramtypes', ctorFn);
        let values = types.map(type => this.resolveInjectable(type));
        var inst = new ctorFn(...values);
        this.cache.set(ctorFn, inst);
        return inst;
    }
    provide(ctorFn, t) {
        if (!ctorFn)
            throw new Error('Attempted to provide a value for a falsey type.');
        if (this.cache.has(ctorFn))
            throw new Error(`Duplicate value provided for ${ctorFn.name || ctorFn}.`);
        this.cache.set(ctorFn, t);
        return this;
    }
}
exports.Injector = Injector;
//# sourceMappingURL=injector.js.map