"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator.throw(value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
class DbImpl {
    constructor(modelFn, model) {
        this.modelFn = modelFn;
        this.model = model;
    }
    create(t) {
        return __awaiter(this, void 0, void 0, function* () {
            if (t instanceof Array) {
                let results = yield this.model.bulkCreate(t);
                return this.wrapResults(results);
            }
            else {
                let result = yield this.model.create(t);
                return this.wrapResult(result);
            }
        });
    }
    findById(id) {
        return __awaiter(this, void 0, void 0, function* () {
            let result = yield this.model.findById(id);
            return result && this.wrapResult(result);
        });
    }
    findOne(query) {
        return __awaiter(this, void 0, void 0, function* () {
            let result = yield this.model.findOne(query);
            return result && this.wrapResult(result);
        });
    }
    findOrCreate(query) {
        return __awaiter(this, void 0, void 0, function* () {
            let [result, created] = yield this.model.findOrCreate(query);
            return [result && this.wrapResult(result), created];
        });
    }
    findAndCountAll(query) {
        return __awaiter(this, void 0, void 0, function* () {
            let results = yield this.model.findAndCountAll(query);
            return { count: results.count, results: this.wrapResults(results.rows) };
        });
    }
    findAll(query) {
        return __awaiter(this, void 0, void 0, function* () {
            let results = yield this.model.findAll(query);
            return this.wrapResults(results);
        });
    }
    all(query) {
        return this.findAll(query);
    }
    count(query) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.model.count(query);
        });
    }
    max(field) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.model.max(field);
        });
    }
    min(field) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.model.min(field);
        });
    }
    sum(field) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.model.sum(field);
        });
    }
    save(t) {
        return __awaiter(this, void 0, void 0, function* () {
            let [affected, results] = yield this.update({ where: { id: t.id } }, t);
            if (affected != 1 || affected != results.length)
                throw new Error(`Tried to save model, but more than one row was affected.`);
            return results[0];
        });
    }
    update(query, replace) {
        return __awaiter(this, void 0, void 0, function* () {
            let [affected, results] = yield this.model.update(replace, query);
            return [affected, this.wrapResults(results)];
        });
    }
    destroy(query) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isT(query))
                query = { where: { id: query.id } };
            return yield this.model.destroy(query);
        });
    }
    isT(query) {
        return !!query.id;
    }
    wrapResult(result) {
        return result;
    }
    wrapResults(results) {
        return results.map(result => this.wrapResult(result));
    }
}
exports.DbImpl = DbImpl;
//# sourceMappingURL=db-impl.js.map