"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator.throw(value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
require('reflect-metadata');
const _ = require('lodash');
const config = require('config');
const Sequelize = require('sequelize');
const metadata_1 = require('../core/metadata');
const db_impl_1 = require('./db-impl');
class OrmReflector {
    constructor(server) {
        this.server = server;
        this.models = new Map();
    }
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            var name = config.get('connections.db.name');
            var port = config.get('connections.db.port');
            var user = config.get('connections.db.user');
            var password = config.get('connections.db.password');
            var host = config.get('connections.db.host');
            var dialect = config.has('connections.db.dialect') ? config.get('connections.db.dialect') : 'mysql';
            this.sql = new Sequelize(name, user, password, {
                host: host,
                dialect: dialect
            });
            this.reflectModels(this.server.meta.models || []);
            yield this.sync();
        });
    }
    sync() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.sql.sync();
        });
    }
    reflectModels(models) {
        for (var q = 0; q < models.length; q++) {
            this.reflectModel(models[q]);
        }
    }
    reflectModel(modelFn) {
        if (this.models.has(modelFn))
            throw new Error(`A model was passed to the orm-reflector twice: ${modelFn}.`);
        var modelProto = modelFn.prototype;
        var meta = Reflect.getOwnMetadata(metadata_1.ModelMetadataSym, modelProto);
        if (!meta)
            throw new Error(`Expecting class with @Model decorator, could not reflect model properties for ${modelProto}.`);
        var tableName = meta.tableName;
        var columns = {};
        var modelOptions = _.cloneDeep(meta);
        var props = Reflect.getOwnMetadata(metadata_1.ModelPropertiesSym, modelProto) || [];
        for (var q = 0; q < props.length; q++) {
            var propName = props[q];
            var propMeta = Reflect.getOwnMetadata(metadata_1.PropMetadataSym, modelProto, propName);
            if (!propMeta)
                throw new Error(`Could not find model property metadata for property ${modelFn}.${propName}.`);
            var columnMeta = _.cloneDeep(propMeta);
            if (propMeta.columnName) {
                delete columnMeta.columnName;
                columnMeta.field = propMeta.columnName;
            }
            columns[propName] = columnMeta;
        }
        let model = this.sql.define(tableName, columns, modelOptions);
        let db = new db_impl_1.DbImpl(modelFn, model);
        this.models.set(modelFn, model);
        modelFn.db = db;
    }
}
exports.OrmReflector = OrmReflector;
//# sourceMappingURL=reflector.js.map