"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator.throw(value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const express = require('express');
const core_1 = require('../core');
const orm_1 = require('../orm');
const services_1 = require('../services');
const router_1 = require('../router');
const http = require('http');
var debug = require("debug")("express:server");
class Server {
    constructor(_meta) {
        this._meta = _meta;
        this._injector = new core_1.Injector();
        this._injector.provide(Server, this);
    }
    get app() {
        return this._app;
    }
    get injector() {
        return this._injector;
    }
    get meta() {
        return this._meta;
    }
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                console.log("Initializing api-server...");
                this._app = express();
                console.log("  Loading database configuration...");
                yield this.reflectOrm();
                console.log("  Starting services...");
                yield this.startServices();
                console.log("  Loading routes...");
                this.reflectRoutes();
            }
            catch (e) {
                console.error("FATAL ERROR: Failed to launch server.");
                console.error(`${e}`);
                return;
            }
            console.log("Serving");
            this.listen();
        });
    }
    reflectOrm() {
        return __awaiter(this, void 0, void 0, function* () {
            this.ormReflector = new orm_1.OrmReflector(this);
            yield this.ormReflector.init();
        });
    }
    startServices() {
        return __awaiter(this, void 0, void 0, function* () {
            this.serviceReflector = new services_1.ServiceReflector(this);
            yield this.serviceReflector.reflectServices(this.meta.services || []);
        });
    }
    reflectRoutes() {
        let router = express.Router();
        this.routerReflector = new router_1.RouterReflector(this, router);
        this.routerReflector.reflectRoutes(this.meta.controllers || []);
        this.app.use(router);
    }
    listen() {
        //create http server
        this.httpServer = http.createServer(this.app);
        //listen on provided ports
        this.httpServer.on("error", (err) => this.onError(err));
        this.httpServer.on("listening", () => this.onListening());
        this.httpServer.listen(this.meta.port);
    }
    onError(error) {
        if (error.syscall !== "listen") {
            throw error;
        }
        var bind = (typeof this.meta.port === "string") ? `pipe ${this.meta.port}` : `port ${this.meta.port}`;
        // handle specific listen errors with friendly messages
        switch (error.code) {
            case "EACCES":
                console.error(`${bind} requires elevated privileges`);
                process.exit(1);
                break;
            case "EADDRINUSE":
                console.error(`${bind} is already in use`);
                process.exit(1);
                break;
            default:
                throw error;
        }
    }
    onListening() {
        var addr = this.httpServer.address();
        var bind = (typeof addr === "string") ? `pipe ${addr}` : `port ${addr.port}`;
        debug(`Listening on ${bind}`);
    }
}
exports.Server = Server;
//# sourceMappingURL=server.js.map