"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator.throw(value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
class ServiceReflector {
    constructor(server) {
        this.server = server;
    }
    reflectServices(services) {
        return __awaiter(this, void 0, void 0, function* () {
            let failures = 0;
            for (var q = 0; q < services.length; q++) {
                let result;
                try {
                    result = yield this.reflectService(services[q]);
                }
                catch (e) {
                    console.error(`Exception occurred when trying to start service: ${services[q].name || services[q]}`);
                    console.error(e);
                    result = false;
                }
                if (!result) {
                    console.error(`    Failed to start service: ${services[q].name || services[q]}`);
                    failures++;
                }
            }
            console.log(`    ${services.length - failures} services started correctly out of ${services.length}`);
        });
    }
    reflectService(serviceFn) {
        return __awaiter(this, void 0, void 0, function* () {
            let service = this.server.injector.resolveInjectable(serviceFn);
            if (typeof service === 'undefined')
                throw new Error(`Failed to inject service: ${serviceFn.name || serviceFn}`);
            if (typeof service.start !== 'undefined') {
                let result = yield service.start();
                if (typeof result === 'boolean' && result === false)
                    return false;
            }
            return true;
        });
    }
}
exports.ServiceReflector = ServiceReflector;
//# sourceMappingURL=reflector.js.map